;Ŀ
; PI Intruders  PI Control Logic 
;

#define PI1                     VARS_PI+18
#define PI2                     VARS_PI+36
#define PI3                     VARS_PI+54
#define PI4                     VARS_PI+72
#define PIExps                  VARS_PI+0
#define PIExpx                  VARS_PI+1
#define PIExpy                  VARS_PI+2
#define PIExpc                  VARS_PI+3

;;PI Data Structure;;;;;;;;;;;;;
#define PIstat  0  ;Status
#define PIx     1  ;X-coordinate
#define PIy     2  ;Y-coordinate
#define PIsi    3  ;Next Start Pos Index
#define PIps    4  ;Pointer to Start Pos Val
#define PIpsr   6  ;Pointer to Start Pos Data
#define PImdcd  8  ;Motion Delay Count-Down
#define PImd    9  ;Motion Delay Value
#define PIfpi   10 ;Falling Pixel Increment
#define PIsway  11 ;Swaying Size
#define PIswayp 12 ;Swaying Polarity
#define PIswayd 13 ;Swaying Delay
#define PIswayc 14 ;Swaying Delay Count
#define PIswayt 15 ;Swaying Track Count

;;PI Initialization;;;;;;;;;;;;;;;;;;;;;;;;;;;
PI_Init:
        ld      hl,PI_Draw
        ld      (DrawProc3),hl
        ld      hl,PI_Timer
        ld      (Timer2+tProc),hl
        ld      hl,PITimerCount
        ld      (Timer2+tCount),hl
        ld      (Timer2+tRecall),hl
        call    PI_SetupPIs
        call    PIExplosion_Init
        ;ld      a,1
        ;ld      (PI1+PIswayp),a
        ;ld      a,2
        ;ld      (PI2+PIswayp),a
        ;ld      a,1
        ;ld      (PI3+PIswayp),a
        ;ld      a,2
        ;ld      (PI4+PIswayp),a
        RET
PI_InitAllVars:
        ld      ix,PI1
        call    PI_xInitVars
        ld      ix,PI2
        call    PI_xInitVars
        ld      ix,PI3
        call    PI_xInitVars
        ld      ix,PI4
        call    PI_xInitVars
        RET
PI_SetupPIs:
        call    PI_InitAllVars
        ld      hl,posx_pi2start
        ld      (PI2+PIpsr),hl
        ld      (PI2+PIps),hl
        ld      hl,posx_pi3start
        ld      (PI3+PIpsr),hl
        ld      (PI3+PIps),hl
        ld      hl,posx_pi4start
        ld      (PI4+PIpsr),hl
        ld      (PI4+PIps),hl
        ld      a,(Level)
        call    Level_PIInit
        RET
PI_Deinit:
        ld      hl,NullFn
        ld      (Timer2+tProc),hl
        ld      hl,NullFn
        ld      (DrawProc3),hl
        call    PIExplosion_Deinit
        RET

;;Global PI Logic;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PI_Timer:
        call    PI_Sway
        call    PI_Fall        
        call    PIExplosion_Elapse
        call    PI_IfAnyHitShip
        RET
PI_Sway:
        ld      ix,PI1
        call    PI_xSwayMotion
        ld      ix,PI2
        call    PI_xSwayMotion
        ld      ix,PI3
        call    PI_xSwayMotion
        ld      ix,PI4
        call    PI_xSwayMotion
        RET
PI_Fall:
        ld      ix,PI1
        call    PI_xFallMotion
        ld      ix,PI2
        call    PI_xFallMotion
        ld      ix,PI3
        call    PI_xFallMotion
        ld      ix,PI4
        call    PI_xFallMotion
        RET
PI_PlaceAll:
        ld      ix,PI1
        call    PI_xPlace
        ld      ix,PI2
        call    PI_xPlace
        ld      ix,PI3
        call    PI_xPlace
        ld      ix,PI4
        call    PI_xPlace
        RET

;;Individual PI Logic;;;;;;;;;;;;;;;;;;;;;;;;;
PI_xSwayMotion:
        ld      a,(ix+PIstat)
        cp      0
        ret     z
        ld      a,(ix+PIsway)
        cp      0
        ret     z
        ld      a,(ix+PIswayc)
        dec     a
        ld      (ix+PIswayc),a
        cp      0
        ret     nz
        ld      a,(ix+PIswayd)
        ld      (ix+PIswayc),a
        jr      PI_xSway
        RET
PI_xSway:
        ld      a,(ix+PIswayp)
        cp      1
        jr      z,PI_xSwayLeft
        cp      2
        jr      z,PI_xSwayRight
        RET
PI_xSway_Inv:
        ld      a,(ix+PIsway)
        ld      (ix+PIswayt),a
        ld      a,(ix+PIswayp)
        cp      1
        jr      z,PI_xSway_Inv1
        cp      2
        jr      z,PI_xSway_Inv2
        RET
PI_xSway_Inv1:
        ld      a,2
        ld      (ix+PIswayp),a
        RET
PI_xSway_Inv2:
        ld      a,1
        ld      (ix+PIswayp),a
        RET
PI_xSwayLeft:
        ld      a,(ix+PIx)
        dec     a
        cp      0
        ld      (ix+PIx),a
        jr      z,PI_xSway_Inv
        ld      a,(ix+PIswayt)
        dec     a
        ld      (ix+PIswayt),a
        cp      0
        ret     nz
        jr      PI_xSway_Inv
        RET
PI_xSwayRight:
        ld      a,(ix+PIx)
        inc     a
        cp      86
        ld      (ix+PIx),a
        jr      z,PI_xSway_Inv
        ld      a,(ix+PIswayt)
        dec     a
        ld      (ix+PIswayt),a
        cp      0
        ret     nz
        jr      PI_xSway_Inv
        RET

PI_xFallMotion:
        ld      a,(ix+PIstat)
        cp      0
        ret     z
        ld      a,(ix+PImdcd)
        dec     a
        ld      (ix+PImdcd),a
        cp      0
        ret     nz
        ld      a,(ix+PImd)
        ld      (ix+PImdcd),a
        jr      PI_xFall
        RET
PI_xFall:
        ld      a,(ix+PIfpi)
        ld      b,a
        ld      a,(ix+PIy)
        add     a,b        
        ld      (ix+PIy),a
        cp      55
        ret     m
        jr      PI_xPlace
        RET
PI_xPlace:
        call    PI_xCompStartPos
        ld      a,d
        ld      (ix+PIx),a
        ld      a,0
        ld      (ix+PIy),a
        ld      a,1
        ld      (ix+PIstat),a
        RET
PI_xInitVars:
        ld      a,0
        ld      (ix+PIstat),a
        ld      (ix+PIsi),a
        ld      a,4
        ld      hl,posx_pi1start
        ld      (ix+PIps+1),h
        ld      (ix+PIps),l
        ld      (ix+PIpsr+1),h
        ld      (ix+PIpsr),l
        ld      a,0
        ld      (ix+PIsway),a
        ld      a,15
        ld      (ix+PIswayd),a
        ld      a,1
        ld      (ix+PImdcd),a
        ld      (ix+PImd),a
        ld      (ix+PIfpi),a
        ld      (ix+PIswayc),a
        ld      (ix+PIswayp),a
        ld      (ix+PIswayt),a
        RET       
PI_xCompStartPos:
        ld      h,(ix+PIps+1)
        ld      l,(ix+PIps+0)
        ld      a,(hl)
        ld      d,a
        ld      a,(ix+PIsi)
        inc     a
        ld      (ix+PIsi),a
        inc     hl
        ld      (ix+PIps+1),h
        ld      (ix+PIps+0),l
        cp      12
        ret     m
        ld      h,(ix+PIpsr+1)
        ld      l,(ix+PIpsr+0)
        ld      (ix+PIps+1),h
        ld      (ix+PIps+0),l
        ld      a,0
        ld      (ix+PIsi),a
        RET
PI_xDestroyedByShot:
        call    ShipShot_Cancel
        call    PI_xDestroyed
        call    Game_Score
        RET
PI_xDestroyed:
        ld      a,(ix+PIx)
        ld      b,a
        ld      a,(ix+PIy)
        ld      c,a
        ld      d,PIExplosionThresh
        call    PIExplosion_PlaceAt
        call    PI_xPlace
        RET
PI_xHitShip:
        call    PI_xDestroyed
        call    Ship_Hit
        RET
        
;;PI Explosion Logic;;;;;;;;;;;;;;;;;;;;;;;;;;
PIExplosion_Init:
        ld      a,0
        ld      (PIExps),a
        ld      hl,PIExplosion_Draw
        ld      (DrawProc6),hl
        RET
PIExplosion_Deinit:
        ld      hl,NullFn
        ld      (DrawProc6),hl
        RET
PIExplosion_Elapse:
        ld      a,(PIExps)
        cp      0
        ret     z
        ld      a,(PIExpc)
        dec     a
        ld      (PIExpc),a
        cp      0
        ret     nz
        ld      a,0
        ld      (PIExps),a
        RET
PIExplosion_PlaceAt:
        ld      a,b
        ld      (PIExpx),a
        ld      a,c
        ld      (PIExpy),a
        ld      a,1
        ld      (PIExps),a
        ld      a,d
        ld      (PIExpc),a
        RET
