; Misc procedures ;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;

; This proc does nothing
NullFn:                            
        RET                        

; This is the timer loop that counts to
; decide when things should happen
StartTimer:
        ld      a,TotalTimers
        ld      (itimer),a
        ld      hl,TIMERS
        ld      (ptimer),hl
TimerLoop:
        ld      ix,(ptimer)
TimerDispatch:
        ld      l,(ix+tRecall)
        ld      h,(ix+tRecall+1)
        ld      a,l
        or      h
        cp      0
        jr      z,TimerLoop_next
        ld      l,(ix+tCount)
        ld      h,(ix+tCount+1)
        dec     hl
        ld      a,l
        or      h
        cp      0
        jr      z,TimerLoop_go
        ld      (ix+tCount),l
        ld      (ix+tCount+1),h
        jr      TimerLoop_next
TimerLoop_go:
        ld      l,(ix+tRecall)
        ld      h,(ix+tRecall+1)
        ld      (ix+tCount),l
        ld      (ix+tCount+1),h
        ld      l,(ix+tProc)
        ld      h,(ix+tProc+1)
        ld      (TimerLoop_call+1),hl
TimerLoop_call:
        call    NullFn
        ld      a,(ProgStat)
        cp      0
        ret     z
TimerLoop_next:
        ld      a,(itimer)
        dec     a
        cp      0
        jr      z,StartTimer        
        ld      (itimer),a
        ld      hl,(ptimer)
        inc     hl
        inc     hl
        inc     hl
        inc     hl
        inc     hl
        inc     hl
        ld      (ptimer),hl
        jr      TimerLoop
Break:
        ld      a,0
        ld      (ProgStat),a
        RET


;; Keyboard reading procedures
KeyPortReset:
        ld      a,kpReset
        out     (1),a
        RET
KeyGetCol:
        out     (1),a
        in      a,(1)
        RET
KeyGetDir:
        call    KeyPortReset
        ld      a,kpGroup1
        call    KeyGetCol
        cp      kpLeft
        jr      z,KGD_left
        cp      kpRight
        jr      z,KGD_right
        ld      a,0
        RET
KGD_left:
        ld      a,kLeft
        RET
KGD_right:
        ld      a,kRight
        RET
KeyGetGroup2:
        call    KeyPortReset
        ld      a,kpGroup2
        call    KeyGetCol
        cp      kpClear
        jr      z,KG2_clear
        ld      a,0
        RET
KG2_clear:
        ld      a,kClear
        RET
KeyGetGroup7:
        call    KeyPortReset
        ld      a,kpGroup7
        call    KeyGetCol
        cp      kp2nd
        jr      z,KG7_2nd
        cp      kpMode
        jr      z,KG7_Mode
        ld      a,0
        RET
KG7_Mode:
        ld      a,kMode
        RET
KG7_2nd:
        ld      a,k2nd
        RET

DispText:
        ld      a,c             ; c = y location                   
        ld      (penrow),a
        ld      a,b             ; b = x location
        ld      (pencol),a
        call    _vputs          ; hl = data
        RET


;; displays real number                              
DispReal:
        ld      a,c             ; c = y location
        ld      (penrow),a
        ld      a,b             ; b = x location       
        ld      (pencol),a
        call    _setxxxxop2
        call    _op2toop1
        ld      a,10
        call    _dispop1a
        RET

;; set contrast of lcd                              
SetContrast:
  	or 	0C0h			
  	call 	_lcd_busy		
        out     (LCDINSTPORT),a
        RET
;; get contrast of lcd        
GetContrast:
        ld      a,0
        or      0C1h
        call    _lcd_busy
        in      a,(LCDINSTPORT)
        RET

;; determines if a number is between two other numbers
Within:
        ld      a,1
        cp      0
        ld      a,d
        add     a,h
        ld      h,a
        ld      a,e
        add     a,l
        ld      l,a
        ld      a,h
        cp      b
        ret     m
        ld      a,b
        cp      d
        ret     m
        ld      a,l
        cp      c
        ret     m
        ld      a,c
        cp      e
        ret     m
        ld      a,0
        cp      0
        RET


; procedure that draws a sprite
; acquired from another source
; not sure who to credit for this routine
DrawSprite:
        push    hl
        ld      hl,0            
        ld      d,0
        add     hl,de
        add     hl,de
        add     hl,de
        add     hl,hl
        add     hl,hl
        ld      d,0             
        ld      e,a
        srl     e
        srl     e
        srl     e
        add     hl,de
        ld      de,8e29h
        add     hl,de           
        ld      b,00000111b     
        and     b
        cp      0               
        jp      z,SPALN
        pop     ix       
        ld      d,a      
        ld      e,8      
SPLLP:  ld      b,(ix+0) 
        ld      c,0      
        push    de
SPSHL:  srl     b
        rr      c
        dec     d
        jp      nz,SPSHL
        pop     de
        ld      a,b    
        or      (hl)
        ld      (hl),a
        inc     hl
        ld      a,c
        or      (hl)
        ld      (hl),a
        ld      bc,11  
        add     hl,bc
        inc     ix     
        dec     e
        jp      nz,SPLLP
        RET
SPALN:                         
        pop     de             
        ld      b,8
SPALP:  ld      a,(de)
        or      (hl)           
        ld      (hl),a
        inc     de
        push    bc
        ld      bc,12
        add     hl,bc
        pop     bc
        djnz    SPALP
        RET
